/*
   +----------------------------------------------------------------------+
   | HipHop for PHP                                                       |
   +----------------------------------------------------------------------+
   | Copyright (c) 2010-present Facebook, Inc. (http://www.facebook.com)  |
   | Copyright (c) 1997-2010 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
   | available through the world-wide-web at the following url:           |
   | http://www.php.net/license/3_01.txt                                  |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
   +----------------------------------------------------------------------+
*/

#include "hphp/runtime/ext/debugger/ext_debugger.h"
#include "hphp/runtime/ext/sockets/ext_sockets.h"
#include "hphp/runtime/ext/vsdebug/debugger.h"
#include "hphp/runtime/ext/vsdebug/ext_vsdebug.h"
#include "hphp/runtime/debugger/debugger.h"
#include "hphp/runtime/debugger/debugger_proxy.h"
#include "hphp/runtime/vm/jit/translator-inline.h"
#include "hphp/runtime/vm/unwind.h"

namespace HPHP {
///////////////////////////////////////////////////////////////////////////////
TRACE_SET_MOD(debugger);

using namespace Eval;

struct DebuggerExtension final : Extension {
  DebuggerExtension() : Extension("debugger", NO_EXTENSION_VERSION_YET) {}
  void moduleInit() override {
    HHVM_NAMED_FE(__SystemLib\\debugger_get_info, HHVM_FN(debugger_get_info));
    HHVM_FE(hphpd_auth_token);
    HHVM_FE(hphpd_break);
    HHVM_FE(hphp_debugger_attached);
    HHVM_FE(hphp_debug_break);
    loadSystemlib();
  }
} s_debugger_extension;

///////////////////////////////////////////////////////////////////////////////

String HHVM_FUNCTION(hphpd_auth_token) {
  TRACE(5, "in f_hphpd_auth_token()\n");
  if (auto proxy = Debugger::GetProxy()) {
    return String(proxy->requestAuthToken());
  }

  return String();
}

void HHVM_FUNCTION(hphpd_break, bool condition /* = true */) {
  TRACE(5, "in f_hphpd_break()\n");
  if (!RuntimeOption::EnableHphpdDebugger || !condition ||
      g_context->m_dbgNoBreak) {
    TRACE(5, "bail !%d || !%d || %d\n", RuntimeOption::EnableHphpdDebugger,
          condition, g_context->m_dbgNoBreak);
    return;
  }
  VMRegAnchor _;
  Debugger::InterruptVMHook(HardBreakPoint);
  if (RuntimeOption::EvalJit && DEBUGGER_FORCE_INTR) {
    TRACE(5, "switch mode\n");
    throw VMSwitchModeBuiltin();
  }
  TRACE(5, "out f_hphpd_break()\n");
}

// Hard breakpoint for the VSDebug extension debugger.
bool HHVM_FUNCTION(hphp_debug_break, bool condition /* = true */) {
  if (!condition) {
    return false;
  }

  auto debugger = HPHP::VSDEBUG::VSDebugExtension::getDebugger();
  if (debugger == nullptr) {
    return false;
  }

  return debugger->onHardBreak();
}

// Quickly determine if a debugger is attached to the current thread.
bool HHVM_FUNCTION(hphp_debugger_attached) {
  if (RuntimeOption::EnableHphpdDebugger && (Debugger::GetProxy() != nullptr)) {
    return true;
  }

  auto debugger = HPHP::VSDEBUG::VSDebugExtension::getDebugger();
  return (debugger != nullptr && debugger->clientConnected());
}

const StaticString
  s_clientIP("clientIP"),
  s_clientPort("clientPort");

Array HHVM_FUNCTION(debugger_get_info) {
  Array ret(Array::Create());
  if (!RuntimeOption::EnableHphpdDebugger) return ret;
  DebuggerProxyPtr proxy = Debugger::GetProxy();
  if (!proxy) return ret;
  Variant address;
  Variant port;
  if (proxy->getClientConnectionInfo(ref(address), ref(port))) {
    ret.set(s_clientIP, address);
    ret.set(s_clientPort, port);
  }
  return ret;
}

///////////////////////////////////////////////////////////////////////////////
}
